import java.io.File;
import java.util.ArrayList;

/**
 * Das Modell einer Registermaschine.
 * 
 * @author Bernd Gramlich
 * @version 2015-01-18
 */
public interface MaschinenModel
{
    /**
     * Beginne ein neues Programm.
     */
    void neu ();

    /**
     * Lade das Programm aus der Programmdatei.
     * 
     * @param programmDatei Die Datei, aus der das Programm geladen werden soll.
     */
    void ladeProgramm (File programmDatei);
     
     /**
     * Speichere das Programm in die Programmdatei.
     * 
     * @param programmDatei Die Datei, in die das Programm gespeichert werden soll.
     */
    boolean speichereProgramm (File programmDatei);
    
    /**
     * Lass das Programm laufen.
     */
    void laufe ();
    
    /**
     * Lass das Programm schrittweise laufen.
     */
    void laufeSchrittweise ();

    /**
     * Unterbrich das Programm.
     */
    void unterbrich ();

    /**
     * Setze das Programm fort.
     */
    void setzeFort ();

    /**
     * Brich das Programm ab.
     */
    void brichAb ();
    
    /**
     * Schreibe einen Befehl in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die der Befehl geschrieben werden soll.
     * @param kommando Der Befehl, der in die Programmzeile geschrieben werden soll.
     */
    void setzeBefehl (int zeile, Befehl kommando);
    
    /**
     * Schreibe ein Argument in eine Programmzeile.
     * 
     * @param zeile Die Programmzeile, in die das Argument geschrieben werden soll.
     * @param argument Das Argument, das in die Programmzeile geschrieben werden soll.
     * 
     * @return True genau dann, wenn das Argument korrekt war.
     */
    boolean setzeArgument (int zeile, int argument);
    
    /**
     * Schreibe eine Zahl in eine Speicherzelle.
     * 
     * @param zelle Die Speicherzelle, in die die Zahl geschrieben werden soll.
     * @param zahl Die Zahl, die in die Speicherzelle geschrieben werden soll.
     * 
     * @return True genau dann, wenn die Zahl korrekt war.
     */
    boolean setzeZelle (int zelle, int zahl);

    /**
     * Nenne die Programmbefehle.
     * 
     * @return Die Liste der Programmbefehle.
     */
    ArrayList<Befehl> gibProgramm ();

    /**
     * Nenne die Argumente der Programmbefehle.
     * 
     * @return Die Liste der Argumente der Programmbefehle.
     */
    ArrayList<Integer> gibArgumente ();

    /**
     * Nenne die Speicherbelegung.
     * 
     * @return Die Liste der Zelleninhalte.
     */
    ArrayList<Integer> gibSpeicherbelegung ();
    
    /**
     * Nenne die Nummer des aktuellen Befehls.
     * 
     * @return Die Nummer des aktuellen Befehls.
     */
    int gibBefehlsnummer ();
    
    /**
     * Nenne die Nummer der aktuellen Speicherzelle.
     * 
     * @return Die Nummer der aktuellen Speicherzelle.
     */
    int gibZellennummer ();
    
    /**
     * Nenne den Inhalt der aktuellen Speicherzelle.
     * 
     * @return Der Inhalt der aktuellen Speicherzelle.
     */
    int gibZelleninhalt ();
    
    /**
     * Nenne den Inhalt des Akkumulators.
     * 
     * @return Der Inhalt des Akkumulators.
     */
    int gibAkkumulatorinhalt ();
    
    /**
     * Nenne den Inhalt des Befehlszaehlers.
     * 
     * @return Der Inhalt des Befehlszaehlers.
     */
    int gibBefehlszaehlerinhalt ();
    
    /**
     * Speichere einen Maschinenbeobachter.
     * 
     * @param b Der Maschinenbeobachter.
     */
    void speichereAlsBeobachter (MaschinenBeobachter b);

    /**
     * Loesche einen Positionsbeobachter.
     * 
     * @param b Der Positionsbeobachter.
     */
    void loescheAlsBeobachter (MaschinenBeobachter b);
}